
/* 
 * OverAll.m -- Implementation file for the OverAll class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "OverAll.h"
#import "nxlcau4t.h"
#import "rec.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>
#import <time.h>

extern int  co, dn;			/* connectivity, dimension */

NXColor colst[KK];
char dfclr[KK][12] = {"1.0,0.0,0.0",
		      "0.0,1.0,0.0",
		      "0.0,0.0,1.0",
		      "0.2,0.3.0.5"
		      };

NXDefaultsVector defaults = {
   {"well0","1.0,0.0,0.0"},
   {"well1","0.0,1.0,0.0"},
   {"well2","0.0,0.0,1.0"},
   {"well2","0.2,0.3.0.5"},
   {NULL,NULL}
   };

char colorwells[KK][6]={"well0","well1","well2","well3"};

extern struct fptbl dtbl[];
extern Inst xprg[XLEN];
extern char idef[][30];


@implementation OverAll

/* ---------------  A P P   I N I T I A L I Z A T I O N   ----------------- */

- appWillInit:sender {printf("appWillInit\n"); return self;}

- appDidInit:sender {
char c[40], ch[WNML], *df ;
int    i, o;
double red, green, blue;
struct tm  *timeptr;
time_t  secsnow;
  printf("appDidInit = nxlcau4t\n");
  rule4t();		 /* printf("rule4t\n"); */
  co=1;
  dn=1;			/* one-dimensional */
  asctobin();
  ttzero();
//  stzero();
  gfzero();
  [rulePanel setAllRules: self];
//  [rulePanel setTotRules: self];
//  liferule();
//  conlif();
//  golta();
//  ranplacor();
  asctobin();

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(c,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: c];

   [rijnView rijnReset: self];

   NXRegisterDefaults("NXLCAU4TDefaults", defaults);
   NXUpdateDefaults();
    for(i=0; i <KK; i++) {
     df=NXGetDefaultValue("NXLCAU4TDefaults",colorwells[i]);
     if (df==NULL) {
	NXWriteDefault("NXLCAU4TDefaults",colorwells[i],dfclr[i]);
	df=&dfclr[i];
	}
     sscanf(df,"%6lf,%6lf,%6lf",&red,&green,&blue); /* don't use %F */
     colst[i]=NXConvertRGBToColor(red,green,blue);
     printf("%s: %s\n",colorwells[i],df);
     }

  [[NXColorPanel sharedInstance: YES] setContinuous: YES];
  [self updateColors];

   /*  initialize r_vst with selected standard procedures */
   rec_c(idef[0], xprg[0], XLEN, dtbl);
   r_vst['a']=&xprg[0];

   [browser getTitleFromPreviousColumn: YES];
   [browser setDoubleAction: @selector(copyAndRunREC:)];
   [browser setPath: "/demonstrations"];

return self;
}
 
- appWillTerminate:sender {printf("appWillTerminate\n"); return self;}

/* ----------------------------- G E N E R A L ----------------------------- */

- saveColors:sender {char c[32]; int i;
    colst[0]=[inCol0 color];
    colst[1]=[inCol1 color];
    colst[2]=[inCol2 color];
    colst[3]=[inCol3 color];
  for (i=0; i<KK; i++) {
    sprintf(c,"%6.3f, %6.3f, %6.3f",
	NXRedComponent(colst[i]),
  	NXGreenComponent(colst[i]),
	NXBlueComponent(colst[i]));
    NXWriteDefault("NXLCAU4TDefaults",colorwells[i],c);
	};
return self;
}

- revertColors:sender 
{char *df; int i; double red, green, blue;
  for (i=0; i<KK; i++) {
    df=NXGetDefaultValue("NXLCAU4TDefaults",colorwells[i]);
    if (df==NULL) {printf("null df = %s\n",df); return self;}
    sscanf(df,"%6lf,%6lf,%6lf",&red,&green,&blue); /* don't use %F */
    colst[i]=NXConvertRGBToColor(red,green,blue);
    printf("%s: %s\n",colorwells[i],df);
    }
  [self updateColors];
return self;
}

- updateColors {
printf("updateColors\n");
	[inCol0 setColor: colst[0]];
	[inCol1 setColor: colst[1]];
	[inCol2 setColor: colst[2]];
	[inCol3 setColor: colst[3]];
return self;
}

- windowDidBecomeKey:sender {
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyWells:)];
	[menuCopy setEnabled: YES];
return self;
}

- windowDidResignKey:sender {[menuCopy setEnabled: NO]; return self;}

- copyWells:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theWellBox getBounds: &zbounds];
  [theWellBox writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

/* - - - - - - - -  R E C   L I N E   C O N F I G U R A T I O N  - - - - - */
 
- useRECLine:sender {
  [RECPanel makeKeyAndOrderFront: sender];
  [RECValue setStringValue: "*"];
 return self;
}

- runRECLine:sender; {
  [RECValue setStringValue: " "];
  if (rec([sender stringValue])) {
     [RECValue setStringValue: "T"];
     [rulePanel setAllRules: self];
     }
  else
    [RECValue setStringValue: "F"];
  return self;
}

/* << S E T - D I M E N S I O N - A N D - L O A D - A C C O R D I N G L Y >> */

- dimenChoice:sender {
   if (dimenPrefs==nil) {
      if (![NXApp loadNibSection: "overall.nib" owner: self withNames: NO]) 
	{return nil;}
      [dimenMenu setEmptySelectionEnabled: YES];
      [dimenMenu selectCellAt: -1: 0];
    }
  [dimenPrefs makeKeyAndOrderFront: sender];
  return self;
}

- setDimension:sender {dn = [sender selectedRow]+1; return self;}

- dbWindow:sender {
printf("deBruinWindow is about to load %d\n",dn);
   switch (dn) {
     case 1:  [rijnView showRijn: sender];
//	      [db2Window performClose: sender];
//	      [db3Window performClose: sender];
	      break;
//     case 2:  [ruinView showRuin: sender];
//	      [db2Window performClose: sender];
//	      [db3Window performClose: sender];
//	      break;
     default: break;}
printf("deBruinWindow is loaded\n");
  return self;
}

@end

/* end OverAll.m */
