
/* 
 * ProbView.m -- Implementation file for the ProbView class 
 *
 * You may freely copy, distribute, and reuse the code in this example. 
 * NeXT disclaims any warranty of any kind, expressed or implied, as to  
 * its fitness for any particular use, nor even that it has a use. 
 *
 */

#import "ProbView.h"
#import "nxlcau4t.h"
#import "nxprob4t.h"
#import "drawingFuncs.h"
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <math.h> 
#import <stdlib.h>
#import <string.h>

extern char ascrule[KK][KK];
extern char auxrule[KK][KK];

extern int  binrule[KK][KK];

int    probBack=YES;
int    drawBasin=NO;
int    drawEProb=NO;
int    drawMeanF=NO;
int    drawSterF=NO;
int    draw2Block=NO;
int    drawNBlk=NO;

extern int    drawview;

extern NXColor colst[KK];

extern double blpr[KK];			/* probability from blocks	*/

double cubic[25][25][25];

extern double stt, sta[5][K3+1];

extern char   arule[DS], arr1[AL], arr2[AL], arr3[AL];

void	asctobin();
void	confr(int vv);
void	conth(double z[25][25][25], double c, int i, int j, int k, int vv);
void	cudem(double f[25][25][25], int l, int m, int n, int v);
void	edshf(int g, int s, int i);
void	evolve();
int	nearest(double x, double y);
void	onegen(int n);
void	penry(int n);
void	pprob();
void	ranlin();
void	ranrul();
void	renewarry();
void	savearry();
void	serms(double x, double y, int l, int m);
void	shortevolve(int m, int n);
double	tdist(int i, int j, int n);
void	totoasc();
void	wru(char *c);

@implementation ProbView

/* ---------------------------  P r o b V i e w  --------------------------- */

- loadProb:sender {id   prbMen;
  if (probWindow==nil) {
    if (![NXApp loadNibSection: "prob.nib" owner: self withNames: NO]) 
	{return nil;}
    prbMen = [[Menu alloc] initTitle: "Probability"];

    [[prbMen 
      addItem: "Stereo" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: stereoWindow];
    [[prbMen 
      addItem: "N-Block" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: blockWindow];
    [[prbMen 
      addItem: "Mean Field" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: probWindow];

    [[sender window] setSubmenu: prbMen forItem: [sender selectedCell]];
    [[sender selectedCell] performClick: sender];
    }
  return self;
  }

- printAProbWindow:sender {
    if (keyWindow==blockWindow)   [self printNBlock:  sender];
    if (keyWindow==probWindow)    [self printMeanF:    sender];
    if (keyWindow==stereoWindow)  [self printStereo:  sender];
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyAProbView:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printAProbWindow:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyAProbView:sender {
    if (keyWindow==blockWindow)   [self copyNBlock: sender];
    if (keyWindow==probWindow)    [self copyProb:   sender];
    if (keyWindow==stereoWindow)  [self copyStereo:  sender];
return self;
}

- copyProb:sender {
NXRect zbounds;
id     pBoard;
  drawview=9;
  probBack=NO;
  drawMeanF=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theProbView getBounds: &zbounds];
  [theProbView writePSCodeInside: &zbounds to: pBoard]; 
  drawMeanF=NO;
  probBack=YES;
return self;
}

- copyStereo:sender {
NXRect zbounds;
id     pBoard;
  drawview=5;
  probBack=NO;
  drawSterF=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theStereoView getBounds: &zbounds];
  [theStereoView writePSCodeInside: &zbounds to: pBoard]; 
  drawSterF=NO;
  probBack=YES;
return self;
}

- copyNBlock:sender {
NXRect zbounds;
id     pBoard;
  drawview=4;
  drawNBlk=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theBlockView getBounds: &zbounds];
  [theBlockView writePSCodeInside: &zbounds to: pBoard]; 
  drawNBlk=NO;
return self;
}

/* -------------------------------- L C A U -------------------------------- */


/* -------------------------------- A R D N -------------------------------- */


/* -------------------------------- R I J N -------------------------------- */


/* -------------------------------- P R O B -------------------------------- */

- drawStereo:sender {
    drawview=5;
    drawSterF=YES;
    [self display];
    drawSterF=NO;
return self;
}

- printStereo:sender {
   drawview=5;
   drawSterF=YES;
   [keyWindow smartPrintPSCode: self];
   drawSterF=NO;
return self;
}

- drawMeanF:sender {
    drawview=5;
    drawMeanF=YES;
    [self display];
    drawMeanF=NO;
return self;
}

- printMeanF:sender {
   drawview=5;
   drawMeanF=YES;
   [keyWindow smartPrintPSCode: self];
   drawMeanF=NO;
return self;
}

- drawNBlock:sender {
  drawview=4;
    drawNBlk=YES;
    [self display];
    drawNBlk=NO;
return self;
}

- printNBlock:sender {
 drawview=4;
   drawNBlk=YES;
   [keyWindow smartPrintPSCode:self];
   drawNBlk=NO;
return self;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - P r o b V i e w - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new ProbView object.  First, an initFrame: message is sent 
 * to super to initialize ProbView as a View.  Next, the ProbView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	NXPoint	spot;
	
	[super initFrame:frameRect];
/*	[self translate: 
		floor(frame.size.width/2) :floor(frame.size.height/2)]; */
	[self setOpaque:YES];
	crossCursor = 
		[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;

}

- setDelegate:anObject
/*
 * Sets the ProbView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}


- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the ProbView's background and axes.  If there are any points,
 * these are drawn too.
 *
 */
{
        char             ch[WNML], c[30];
	unsigned int	 i, j, k, l, n;
	int		 ii, jj, nn, bas;
        double           en, p, q, r, s, t, dp, pee, cue;
	double		 aa[KK], bb[KK], cc[KK], cq[KK];
	double		 bern(), dbern(), tbern(), fbern();
	NXPoint		 *aPoint;
	NXRect		 plotRect;
        struct tm  *timeptr;
        time_t  secsnow;
	
	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (probBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

// drawAxes(bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height);


        if (drawSterF==YES) {
		procur(
			[probEI selectedCol],
			4-[probEI selectedRow],
			[probLevel selectedRow],
			[probView selectedRow]
			);
		};


	if (drawMeanF==YES) {
   wru(ch);
   [ruleNumber setStringValue: ch];

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(c,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: c];

   if (strcmp([[probEI selectedCell] title],"Compose")==0) {
    if ([[probComp cellAt: 0: 0] state]) cudem(cubic,25,25,25,6);
    if ([[probComp cellAt: 1: 0] state]) cudem(cubic,25,25,25,7);
    if ([[probComp cellAt: 2: 0] state]) cudem(cubic,25,25,25,8);
	}
   if (strcmp([[probEI selectedCell] title],"Iterate")==0) {
    if ([[probComp cellAt: 0: 0] state]) cudem(cubic,25,25,25,11);
    if ([[probComp cellAt: 1: 0] state]) cudem(cubic,25,25,25,10);
    if ([[probComp cellAt: 2: 0] state]) cudem(cubic,25,25,25,9);
	}

    p=0.375*[pslid doubleValue];
    l=0;
    if (strcmp([[probView selectedCell] title],"front")==0) l=0;
    if (strcmp([[probView selectedCell] title],"plan")==0)  l=1;
    if (strcmp([[probView selectedCell] title],"side")==0)  l=2;

    if ([[probLevel cellAt: 2: 0] state]) {
	  PSsetlinewidth(0.1);
	  PSsetgray(NX_BLACK);
    for (i=0; i<24; i++) for (j=0; j<24; j++) for (k=0; k<24; k++) {
      conth(cubic,p,i,j,k,l);
	}
		PSstroke();
      }

    if ([[probLevel cellAt: 1: 0] state]) {
	  PSsetlinewidth(0.0);
	  PSsetcmykcolor(0.0,1.0,0.2,0.0);
    for (i=0; i<24; i++) for (j=0; j<24; j++) for (k=0; k<24; k++) {
      conth(cubic,2.0*p,i,j,k,l);
	}
		PSstroke();
     }

    if ([[probLevel cellAt: 0: 0] state]) {
	  PSsetlinewidth(0.0);
	  PSsetcmykcolor(1.0,0.0,0.2,0.0);
    for (i=0; i<24; i++) for (j=0; j<24; j++) for (k=0; k<24; k++) {
      conth(cubic,3.0*p,i,j,k,l);
	}
		PSstroke();
      }

	  PSsetgray(NX_BLACK);
	  confr(l);
	  PSstroke();
            }

	if (drawNBlk==YES) {
          wru(ch);
          [ruleNumber setStringValue: ch];
	  nblcur([[nblokLen selectedCell] title][0]);
	  for (i=0; i<KK; i++) 
		[[nblokAve cellAt: 0: i] setDoubleValue: blpr[i]];
	  }	  

	return self;
}


- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the ProbView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo:width :height];
if (drawview==4||drawview==7) {[self setDrawOrigin: 0: 0]; return self;}
	else [self setDrawOrigin: -floor(width/2): -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the ProbView.  The ProbView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the ProbView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the ProbView, the ProbView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

         double   ex, wy;     
	 int k;

	[crossCursor set];
   	[self getBounds:&plotRect];

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
//                ex=aPoint.x;
//                wy=aPoint.y;
				PSsetgray(NX_BLACK);
//				PSmoveto(ex+12.0,wy);
//				PSarc(ex, wy, 12.0, 0.0, 360.0);
//				PSstroke();
//				cntr[k][0]=ex;
//				cntr[k][1]=wy;
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {looping = NO;}	
   	} while (looping && 
	(theEvent=[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];


	}
	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end ProbView.m */
