
/* SQUL.C					*/
/* Two-Dimensional Square Lattice Life		*/
/* with binary cells, Moore neighborhood	*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/
/* 20 December 1995 - fixed for lcau24 & NeXT	*/

# include "nxlcau4t.h"


# define QL         48	/* array for sample evolution		*/
# define GK        512	/* KK^BD= total number of neighborhoods	*/
# define GH      GK/KK	/* GK/KK= 1/KKth no. of neighborhoods	*/
# define NC	    51  /* number of symmetry classes w/o ctr	*/
# define YS	 NC*KK  /* number of symmetry classes w/ cntr	*/
# define NRR	     3  /* row coordinate for neighborhood disp	*/
# define NRC	    38	/* column coordinate for neighborhood d	*/
# define TRR	     2  /* row coordinate for totalistic rule	*/
# define TRC	     0	/* column coordinate for t'listic  rule	*/
# define AUR	    64  /* row (from bottom) for planar array	*/
# define AUC	    60  /* column for planar array		*/
# define BER	     0  /* row (from bottom) for Bernstein p.	*/
# define BEC	   219  /* column for Bernstein Polynomial	*/
# define LPR	    90  /* row (from bottom) for live pct bar	*/
# define LPC	   219  /* column for live percentage bar 	*/
# define NF	    30	/* lines in live percentage graph	*/
# define RHI	    40  /* height of bar in option 's'		*/
 
# define lcell(i,j) arr1[li+i][lj+j]=1

char    sqrule[YS];	/* evolutionary rule 			*/
char    arule[YS];	/* auxiliary rule flags 		*/
int     li, lj;		/* cursor in cell array 		*/
int	lf, ln;		/* live cells, line number 		*/
int     nr[YS];		/* nbrhd representative of symm class 	*/
int     rs[YS];		/* rule use statistics 			*/
int     gc[GK];		/* Golay code key 			*/
int	sc[NC][BD];
int     wc[NC];		/* size (weight) of symm class 		*/
int     ix0, iy0;	/* origin for pen movement 		*/
double  bp[BD+1];	/* Bernstein coefficients 		*/

squlmain() {
char c;			/* char from keyboard 		*/
char rulstk[5][YS];	/* stack to save rules 		*/
int  i, j;		/* generic indices 		*/
int  rulptr;		/* pointer to rule stack	*/
int  liar[QL][QL];	/* planar array 		*/

//  videomode(COLGRAF);
//  videopalette(YELREGR);

  rulptr=0;
  ln=0; lf=0;
  li=QL/2; lj=QL/2;

  for (i=0; i<YS; i++) rs[i]=0;
  for (i=0; i<YS; i++) arule[i]=' ';

  videocursor(0,18,0);
  golta();
  for (j=1; j<KK; j++) for (i=0; i<GH; i++) gc[i+j*GH]=gc[i]+j*NC;
  for (i=0; i<NC; i++) bp[i]=((double)wc[i]);

  conlif();

  hsqscrrul();
  wsqscrrul();
  lirmenu();
  phrame(3);
//  rahncen(liar);
  screen(liar);
  phrame(3);

  while (0<1) {					/* set up one run */
  lirmenu();
  hsqscrrul();
  wsqscrrul();
//  videocursor(0,0,0);
//  videoputc('?',1);
//c=kbdin();
//  videocursor(0,0,0);
//  videoputc(c,1);
  if (c=='q') break;
  switch (c) {
    case 'g':					/* one cycle of evolution */
      phrame (1);
      onesqgen(liar);
      screen(liar);
      sqlstat(ln%NF,lf);
      ln++;
      break;
    case ':':				/* multiple cycles of evolution */
      rstat();
      phrame (1);
      videoscroll(18,0,24,25,0,0);
      i=0; ln=0;
//      do {
//        i++;
//        videocursor(0,16,0);
//        printf("%5d",i);
//        onesqgen(liar);
//        screen(liar);
//        rstat();
//        if (ln==0) videoscroll(7,26,11,39,0,0);
//        sqlstat(ln,lf);
//        ln++; if (ln==NF) ln=0;
//        } while (!kbdst()); kbdin();
      break;
    case 'M':					/* record rule usage */
      phrame (1);
      videoscroll(18,0,24,25,0,0);
      for (ln=0; ln<40; ln++) {
        for (i=0; i<YS; i++) rs[i]=0;
        videocursor(0,16,0);
        printf("%5d",ln+1);
        onesqgen(liar);
        screen(liar);
        rinci(40-ln);
        sqlstat(ln,lf);
//        if (kbdst()) {kbdin(); break;}
	}
      break;
    case 'o':					/* translate keyboard ch */
//      printf("%o",kbdin());
      break;
    case 's':
      rstat();
      break;
    case 't':					/* probability plot */
      bpanel();
      break;
    case 'l':					/* edit cell array */
      edsqline(liar);
      break;
    case 'r':					/* edit rule */	
      edsqrule();
      videocursor(0,4,0);
      break;
/*        case '#':					/* read stored rule */
/*	    xmenu(NX);
/*	    xtoasc(lim(1,numin(0),NX)-1);
/*	    lirmenu();
/*            break;
/*	case 'D':					/* run through samples */
/*	    for (i=0; i<NX; i++) {
/*	      xmenu(i);
/*	      xtoasc(i);
/*	      ranlin();
/*	      screen(liar);
/*	      }
/*	    videomode(T80X25);
/*	    lirmenu();
/*	    break; */
	case 'd':					/* de Bruijn diagram */
	    eduin();
	    break;
       case 'u':					/* sparse init arry */
	    zrarry(liar);
	    liar[QL/4][QL/4]=1;
	    liar[QL/2+1][QL/2]=1;
	    liar[QL/2-1][QL/2]=1;
	    liar[QL/2][QL/2+1]=1;
	    liar[QL/2][QL/2-1]=1;
	    screen(liar);
	    break;
	case 'v':					/* Conway's Life */
	    conlif();
	    break;
	case 'X':					/* random rule */
	    ransqrul();
	    break;
	case 'x':					/* random rule */
	    for (i=0, j=0; j<YS; j++) {
	      if (i==0) i=rand();
	      if (arule[j]==' ') {sqrule[j]='0'+i%KK; i/=KK;}
	      }
	    break;
	case 'y':					/* random line */
//	    rahncen(liar);
	    screen(liar);
	    break;
	case 'Y':					/* random line */
	    ransqarr(liar);
	    screen(liar);
	    break;
	case 'z':
//	    videomode(COLGRAF);
//	    videopalette(YELREGR);
	    break;
	case 'U':					/* push rule */
	    if (rulptr<5) rulptr++;
	    for (i=0; i<YS; i++) rulstk[rulptr][i]=sqrule[i];
	    break;
	case 'V':					/* pop rule */
	    for (i=0; i<YS; i++) sqrule[i]=rulstk[rulptr][i];
	    if (rulptr>0) rulptr--;
	    break;
	case 'G':					/* fetch rule */
	    for (i=0; i<YS; i++) sqrule[i]=rulstk[rulptr][i];
	    break;
        default: break;
        } /* end switch */
    }	   /* end while */
//  videomode(T80X25);
}	   /* end main */

/* calculate one generation of evolution */
onesqgen(ar,m,n) int  ar[QL][QL], m, n; {int i0, i1, gn, arr2[QL][QL];

/* count neighbors of main square */
phrame(1); /* phase to count neighbors */

for (i0=1; i0<m-1; i0++) {
for (i1=1; i1<n-1; i1++) {
  gn=ar[i0-1][i1+1];
  gn=KK*gn+ar[i0][i1+1];
  gn=KK*gn+ar[i0+1][i1+1];
  gn=KK*gn+ar[i0+1][i1];
  gn=KK*gn+ar[i0+1][i1-1];
  gn=KK*gn+ar[i0][i1-1];
  gn=KK*gn+ar[i0-1][i1-1];
  gn=KK*gn+ar[i0-1][i1];
  if (ar[i0][i1]==1) gn+=GH;
  gn=gc[gn];
  arr2[i0][i1]=gn;
  rs[gn]++;
  }}

/* count neighbors for top edge (i1=0) */

for (i0=1; i0<m-1; i0++) {
  gn=ar[i0-1][1];
  gn=KK*gn+ar[i0][1];
  gn=KK*gn+ar[i0+1][1];
  gn=KK*gn+ar[i0+1][0];
  gn=KK*gn+ar[i0+1][n-1];
  gn=KK*gn+ar[i0][n-1];
  gn=KK*gn+ar[i0-1][n-1];
  gn=KK*gn+ar[i0-1][0];
  if (ar[i0][0]==1) gn+=GH;
  gn=gc[gn];
  arr2[i0][0]=gn;
  rs[gn]++;
  }

/* count neighbors for bottom edge (i1=n-1) */

for (i0=1; i0<m-1; i0++) {
  gn=ar[i0-1][0];
  gn=KK*gn+ar[i0][0];
  gn=KK*gn+ar[i0+1][0];
  gn=KK*gn+ar[i0+1][n-1];
  gn=KK*gn+ar[i0+1][n-2];
  gn=KK*gn+ar[i0][n-2];
  gn=KK*gn+ar[i0-1][n-2];
  gn=KK*gn+ar[i0-1][n-1];
  if (ar[i0][n-1]==1) gn+=GH;
  gn=gc[gn];
  arr2[i0][n-1]=gn;
  rs[gn]++;
  }

/* count neighbors for left edge (i0=0) */

for (i1=1; i1<n-1; i1++) {
  gn=ar[QL-1][i1+1];
  gn=KK*gn+ar[0][i1+1];
  gn=KK*gn+ar[1][i1+1];
  gn=KK*gn+ar[1][i1];
  gn=KK*gn+ar[1][i1-1];
  gn=KK*gn+ar[0][i1-1];
  gn=KK*gn+ar[m-1][i1-1];
  gn=KK*gn+ar[m-1][i1];
  if (ar[0][i1]==1) gn+=GH;
  gn=gc[gn];
  arr2[0][i1]=gn;
  rs[gn]++;
  }

/* count neighbors of right edge (i0=m-1) */

for (i1=1; i1<QL-1; i1++) {
  gn=ar[m-2][i1+1];
  gn=KK*gn+ar[m-1][i1+1];
  gn=KK*gn+ar[0][i1+1];
  gn=KK*gn+ar[0][i1];
  gn=KK*gn+ar[0][i1-1];
  gn=KK*gn+ar[m-1][i1-1];
  gn=KK*gn+ar[m-2][i1-1];
  gn=KK*gn+ar[m-2][i1];
  if (ar[m-1][i1]==1) gn+=GH;
  gn=gc[gn];
  arr2[m-1][i1]=gn;
  rs[gn]++;
  }

/* count neighbors for upper left hand corner */

  gn=ar[QL-1][1];
  gn=KK*gn+ar[0][1];
  gn=KK*gn+ar[1][1];
  gn=KK*gn+ar[0][1];
  gn=KK*gn+ar[1][n-1];
  gn=KK*gn+ar[0][n-1];
  gn=KK*gn+ar[n-1][QL-1];
  gn=KK*gn+ar[n-1][0];
  if (ar[0][0]==1) gn+=GH;
  gn=gc[gn];
  arr2[0][0]=gn;
  rs[gn]++;

/* count neighbors for upper right hand corner */

  gn=ar[QL-2][1];
  gn=KK*gn+ar[m-1][1];
  gn=KK*gn+ar[0][1];
  gn=KK*gn+ar[0][0];
  gn=KK*gn+ar[0][n-1];
  gn=KK*gn+ar[m-1][n-1];
  gn=KK*gn+ar[m-2][n-1];
  gn=KK*gn+ar[m-2][0];
  if (ar[m-1][0]==1) gn+=GH;
  gn=gc[gn];
  arr2[QL-1][0]=gn;
  rs[gn]++;

/* count neighbors for lower left hand corner */

  gn=ar[m-1][0];
  gn=KK*gn+ar[0][0];
  gn=KK*gn+ar[1][0];
  gn=KK*gn+ar[m-1][n-2];
  gn=KK*gn+ar[1][n-2];
  gn=KK*gn+ar[0][n-2];
  gn=KK*gn+ar[m-1][n-2];
  gn=KK*gn+ar[m-1][n-1];
  if (ar[0][n-1]==1) gn+=GH;
  gn=gc[gn];
  arr2[0][n-1]=gn;
  rs[gn]++;

/* count neighbors for lower right hand corner */

  gn=ar[m-2][0];
  gn=KK*gn+ar[m-1][0];
  gn=KK*gn+ar[0][0];
  gn=KK*gn+ar[0][n-1];
  gn=KK*gn+ar[0][n-2];
  gn=KK*gn+ar[m-1][n-2];
  gn=KK*gn+ar[m-2][n-2];
  gn=KK*gn+ar[m-2][n-1];
  if (ar[m-1][n-1]==1) gn+=GH;
  gn=gc[gn];
  arr2[m-1][n-1]=gn;
  rs[gn]++;

phrame(2); /* phase to update cells */
lf=0;
for (i0=0; i0<m; i0++) {
for (i1=0; i1<n; i1++) {
  ar[i0][i1]=sqrule[arr2[i0][i1]]-'0';
  if (ar[i0][i1]==1) lf++;
}}
phrame(3); /* phase to show results  */
}	  /* end subroutine onesqgen */
    
/* display a screen of evolution */
screen(ar,m,n) int  ar[QL][QL], m, n; {int i0, i1;
  for (i1=0; i1<m; i1++) {
  for (i0=0; i0<n; i0++) {
    adot(i0,i1,ar[i0][i1]);
    }}
}

/* display a phrame around the cell array */
phrame(l) int l; {int j;
for (j=0; j<QL; j++) {
  adot(0,j,l);
  adot(QL-1,j,l);
  adot(j,0,l);
  adot(j,QL-1,l);
  }
}

/* make a phrame for the fixpoint graph  */
/* n     = dimension of phrame           */
gsqfram(n) int n; {int i;
for (i=0; i<=n; i++) {
  bdot(i,0,0);
  bdot(i,n,0);
  bdot(n,i,0);
  bdot(0,i,0);
  }
for (i=0; i<=n; i+=10) {
  bdot(i,0,3);
  bdot(i,n,3);
  bdot(n,i,3);
  bdot(0,i,3);
  }
}

/* make a grid for the fixpoint graph  */
/* n     = dimension of grid           */
gsqgrid(n) int n; {int i, j;
for (i=0; i<=n; i+=10) for (j=0; j<=n; j+=10) bdot(i,j,3);}

/* put a diagonal in the fixpoint graph */
gsqdiag(n) int n; {int i; for (i=0; 2*i<n; i++) bdot(2*i,2*i,3);}

/* graph incidence of rule usage */
/* beware limited count capacity  */
rinci(j) int j; {int i; for (i=0; i<YS; i++) sdot(i,j,rs[i]==0?1:3);}

/* graph statistics of rule usage */
/* beware limited count capacity  */
/* large counts look negative     */
rstat() {int i, j, m, n; double s, t;
for (i=0, s=0.0; i<YS; i++) s+=(double)rs[i];
if  (s<1.0) {for (i=0; i<YS; i++) rs[i]=0; return;}
for (i=0; i<YS; i++) {
  t=500.0*(((double)rs[i])/s);	/* full scale = RHI/500 population */
  if (t>=32000.0) t=50.0;	/* larger looks negative */
  n=(int)t;
  sdot(i,RHI,0);
  m=RHI; if (n>RHI) {n=RHI; m=RHI+1;}
  if (n>0) for (j=0; j<n; j++) sdot(i,j,1);
  for (j=n; j<m; j++) sdot(i,j,2);
  rs[i]=0;
  }
}

/* graph fraction of live cells */
sqlstat(y,n) int y, n; {int i, k;
k=(int)(100.0*(((double)(n))/((double)(QL*QL))));
if (k<=0) k=1; if (k>=100) k=99;
for (i=0; i<k; i++) ldot(i,y,1);
for (i=k; i<101; i++) ldot(i,y,2);
for (i=0; i<101; i++) ldot(i,y+1,0);
for (i=0; i<101; i+=10) ldot(i,y+1,3);
}

/* generate the Golay code table */
golta() {
int i, j, k;
int i0, i1, i2, i3, i4, i5, i6, i7;
k=0;
for (i7=0; i7<KK; i7++) {
for (i6=0; i6<KK; i6++) {
for (i5=0; i5<KK; i5++) {
for (i4=0; i4<KK; i4++) {
for (i3=0; i3<KK; i3++) {
for (i2=0; i2<KK; i2++) {
for (i1=0; i1<KK; i1++) {
for (i0=0; i0<KK; i0++) {
i=binq(i0,i1,i2,i3,i4,i5,i6,i7);
j=binq(i2,i3,i4,i5,i6,i7,i0,i1);
if (j<i) i=j;
j=binq(i4,i5,i6,i7,i0,i1,i2,i3);
if (j<i) i=j;
j=binq(i6,i7,i0,i1,i2,i3,i4,i5);
if (j<i) i=j;
j=binq(i0,i7,i6,i5,i4,i3,i2,i1);
if (j<i) i=j;
j=binq(i6,i5,i4,i3,i2,i1,i0,i7);
if (j<i) i=j;
j=binq(i4,i3,i2,i1,i0,i7,i6,i5);
if (j<i) i=j;
j=binq(i2,i1,i0,i7,i6,i5,i4,i3);
if (j<i) i=j;
j=binq(i0,i1,i2,i3,i4,i5,i6,i7);
if (i==j) {
  gc[i]=k;
  sc[k][i0]++; sc[k][i1]++; sc[k][i2]++; sc[k][i3]++;
  wc[k]=1;
  nr[k]=j;
  k++;
} else {
  gc[j]=gc[i]; wc[gc[i]]++;
  }
/*
printf("  %2X, gc[%3d]=%3d, k=%3d, (%d,%d)\n",j,j,i,k,i0+i2+i4+i6,i1+i3+i5+i7);
*/
}}}}}}}}
/*
printf("no. cl. = %3d",k);
*/
}

/* calculate binary equivalent */
int binq(i0,i1,i2,i3,i4,i5,i6,i7)
int i0, i1, i2, i3, i4, i5, i6, i7; {
return i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*(i6+KK*i7))))));}

/* calculate Golay class of a cell */
int golcl(i0,i1,ar) int  i0, i1, ar[QL][QL]; {int gn;
gn=ar[i0-1][i1+1];
gn=KK*gn+ar[i0][i1+1];
gn=KK*gn+ar[i0+1][i1+1];
gn=KK*gn+ar[i0+1][i1];
gn=KK*gn+ar[i0+1][i1-1];
gn=KK*gn+ar[i0][i1-1];
gn=KK*gn+ar[i0-1][i1-1];
gn=KK*gn+ar[i0-1][i1];
if (ar[i0][i1]==1) gn+=GH;
gn=gc[gn];
return gn;
}

/* plot self-consistent probability graph */
//bpanel()  {sqbernc(); gsqgrid(100); gsqdiag(100); bsqgraf(BD,100);}

/* generate Bernstein coefficients */
sqbernc() {int i;
for (i=0; i<BD+1; i++) bp[i]=0.0;
if (sqrule[0]=='1') bp[0]+=1.0;
if (sqrule[1]=='1') bp[1]+=4.0;
if (sqrule[2]=='1') bp[1]+=4.0;
if (sqrule[3]=='1') bp[2]+=8.0;
if (sqrule[4]=='1') bp[2]+=4.0;
if (sqrule[5]=='1') bp[3]+=4.0;
if (sqrule[6]=='1') bp[2]+=8.0;
if (sqrule[7]=='1') bp[2]+=4.0;
if (sqrule[8]=='1') bp[3]+=8.0;
if (sqrule[9]=='1') bp[3]+=8.0;
if (sqrule[10]=='1') bp[3]+=4.0;
if (sqrule[11]=='1') bp[4]+=8.0;
if (sqrule[12]=='1') bp[2]+=2.0;
if (sqrule[13]=='1') bp[3]+=8.0;
if (sqrule[14]=='1') bp[3]+=4.0;
if (sqrule[15]=='1') bp[4]+=8.0;
if (sqrule[16]=='1') bp[4]+=4.0;
if (sqrule[17]=='1') bp[5]+=4.0;
if (sqrule[18]=='1') bp[2]+=2.0;
if (sqrule[19]=='1') bp[3]+=8.0;
if (sqrule[20]=='1') bp[3]+=4.0;
if (sqrule[21]=='1') bp[4]+=4.0;
if (sqrule[22]=='1') bp[3]+=4.0;
if (sqrule[23]=='1') bp[3]+=4.0;
if (sqrule[24]=='1') bp[4]+=8.0;
if (sqrule[25]=='1') bp[4]+=8.0;
if (sqrule[26]=='1') bp[4]+=8.0;
if (sqrule[27]=='1') bp[5]+=8.0;
if (sqrule[28]=='1') bp[4]+=4.0;
if (sqrule[29]=='1') bp[4]+=8.0;
if (sqrule[30]=='1') bp[4]+=4.0;
if (sqrule[31]=='1') bp[5]+=8.0;
if (sqrule[32]=='1') bp[4]+=4.0;
if (sqrule[33]=='1') bp[5]+=8.0;
if (sqrule[34]=='1') bp[5]+=8.0;
if (sqrule[35]=='1') bp[5]+=4.0;
if (sqrule[36]=='1') bp[6]+=8.0;
if (sqrule[37]=='1') bp[4]+=1.0;
if (sqrule[38]=='1') bp[5]+=4.0;
if (sqrule[39]=='1') bp[5]+=4.0;
if (sqrule[40]=='1') bp[6]+=4.0;
if (sqrule[41]=='1') bp[5]+=8.0;
if (sqrule[42]=='1') bp[6]+=8.0;
if (sqrule[43]=='1') bp[6]+=2.0;
if (sqrule[44]=='1') bp[7]+=4.0;
if (sqrule[45]=='1') bp[4]+=1.0;
if (sqrule[46]=='1') bp[5]+=4.0;
if (sqrule[47]=='1') bp[6]+=4.0;
if (sqrule[48]=='1') bp[6]+=2.0;
if (sqrule[49]=='1') bp[7]+=4.0;
if (sqrule[50]=='1') bp[8]+=1.0;
if (sqrule[51]=='1') bp[1]+=1.0;
if (sqrule[52]=='1') bp[2]+=4.0;
if (sqrule[53]=='1') bp[2]+=4.0;
if (sqrule[54]=='1') bp[3]+=8.0;
if (sqrule[55]=='1') bp[3]+=4.0;
if (sqrule[56]=='1') bp[4]+=4.0;
if (sqrule[57]=='1') bp[3]+=8.0;
if (sqrule[58]=='1') bp[3]+=4.0;
if (sqrule[59]=='1') bp[4]+=8.0;
if (sqrule[60]=='1') bp[4]+=8.0;
if (sqrule[61]=='1') bp[4]+=4.0;
if (sqrule[62]=='1') bp[5]+=8.0;
if (sqrule[63]=='1') bp[3]+=2.0;
if (sqrule[64]=='1') bp[4]+=8.0;
if (sqrule[65]=='1') bp[4]+=4.0;
if (sqrule[66]=='1') bp[5]+=8.0;
if (sqrule[67]=='1') bp[5]+=4.0;
if (sqrule[68]=='1') bp[6]+=4.0;
if (sqrule[69]=='1') bp[3]+=2.0;
if (sqrule[70]=='1') bp[4]+=8.0;
if (sqrule[71]=='1') bp[4]+=4.0;
if (sqrule[72]=='1') bp[5]+=4.0;
if (sqrule[73]=='1') bp[4]+=4.0;
if (sqrule[74]=='1') bp[4]+=4.0;
if (sqrule[75]=='1') bp[5]+=8.0;
if (sqrule[76]=='1') bp[5]+=8.0;
if (sqrule[77]=='1') bp[5]+=8.0;
if (sqrule[78]=='1') bp[6]+=8.0;
if (sqrule[79]=='1') bp[5]+=4.0;
if (sqrule[80]=='1') bp[5]+=8.0;
if (sqrule[81]=='1') bp[5]+=4.0;
if (sqrule[82]=='1') bp[6]+=8.0;
if (sqrule[83]=='1') bp[5]+=4.0;
if (sqrule[84]=='1') bp[5]+=8.0;
if (sqrule[85]=='1') bp[6]+=8.0;
if (sqrule[86]=='1') bp[6]+=4.0;
if (sqrule[67]=='1') bp[7]+=8.0;
if (sqrule[88]=='1') bp[5]+=1.0;
if (sqrule[89]=='1') bp[6]+=4.0;
if (sqrule[90]=='1') bp[6]+=4.0;
if (sqrule[91]=='1') bp[7]+=4.0;
if (sqrule[92]=='1') bp[6]+=8.0;
if (sqrule[93]=='1') bp[7]+=8.0;
if (sqrule[94]=='1') bp[7]+=2.0;
if (sqrule[95]=='1') bp[8]+=4.0;
if (sqrule[96]=='1') bp[5]+=1.0;
if (sqrule[97]=='1') bp[6]+=4.0;
if (sqrule[98]=='1') bp[7]+=4.0;
if (sqrule[99]=='1') bp[7]+=2.0;
if (sqrule[100]=='1') bp[8]+=4.0;
if (sqrule[101]=='1') bp[9]+=1.0;
}

/* graph Bernstein polynomial */
bsqgraf(k,n) int k, n; {int i; double sqbern(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  bdot(i,(int)(en*sqbern(p,k)),1);
  }
}

/* evaluate the nth degree Bernstein polynomial at point p */
double sqbern(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* adot(i,j,l) - place dot on automaton screen at (i,j) in color l */
adot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* bdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
// bdot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* ldot(i,j,l) - place dot on live probability  screen at (i,j) in color l */
ldot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* sdot(i,j,l) - place dot on statistics screen at (i,j) in color l */
sdot(i,j,l) int i, j, l; {videodot(j,2+2*i,l);}

/* clear the cell array */
zrarry(ar) int  ar[QL][QL]; {int i0, i1;
for (i0=0; i0<QL; i0++)
for (i1=0; i1<QL; i1++)
  ar[i0][i1]=0;
}

/* display rule menu */
lirmenu() {
    sqscrrul(TRR,TRC);
    }

/* display rule number by sum in color */
sqscrrul(y,x) int x, y; {int i, n;
//n=YS/3;
//videocursor(0,y,x);
//  scrstr("  ");
//  for (i=0;   i<n;   i++) resch(i);
//videocursor(0,y+1,x);
//  scrstr("  ");
//  for (i=n;   i<2*n; i++) resch(i);
//videocursor(0,y+2,x);
//  scrstr("  ");
//  for (i=2*n; i<3*n; i++) resch(i);
//videocursor(0,y,x);
}

/* display rule number as a hexadecimal */
hsqscrrul() {int i; long int k;
  videocursor(0,0,0);
  printf("    ");
  for (i=0,  k=0; i<6;  i++) k=2*k+sqrule[i]-'0';
  printf("%2.2lX",k);
  for (i=6,  k=0; i<38; i++) k=2*k+sqrule[i]-'0';
  printf("%8.8lX",k);
  for (i=38, k=0; i<70; i++) k=2*k+sqrule[i]-'0';
  printf("%8.8lX",k);
  for (i=70, k=0; i<YS; i++) k=2*k+sqrule[i]-'0';
  printf("%8.8lX",k);
  videocursor(0,0,0);
  }

/* display Wolfram rule number of cross section */
wsqscrrul() {int i;
  i=sqrule[101]-'0';
  i=2*i+sqrule[68]-'0';
  i=2*i+sqrule[48]-'0';
  i=2*i+sqrule[10] -'0';
  i=2*i+sqrule[68]-'0';
  i=2*i+sqrule[63]-'0';
  i=2*i+sqrule[10] -'0';
  i=2*i+sqrule[0] -'0';
  videocursor(0,0,32);
  printf("     ");
  videocursor(0,0,32);
  printf("[%2d]",i);
  videocursor(0,0,0);
  }

/* edit the rule */
edsqrule() {char c; int  i, j, n;
  n=YS/3; i=0; j=0;
  while (0<1) {
    shon((j*n+i)/(YS/2),nr[(j*n+i)%(YS/2)]);
    videocursor(0,TRR+j,TRC+i+2);
    videoputc(sqrule[j*n+i],3);
    videocursor(0,TRR+j,TRC+i+2);
//    c=kbdin();
      if (c=='\015') break;				/* carriage return */
      switch (c) {
        case '0': case '1': 				/* state = 0,1 */
          sqrule[j*n+i]=c;
	  resch(j*n+i);
          i++; if (i%n==0) {i=0; j++; if (j>2) j=0;}
	  break;
        case ' ': case '\315':				/* space = advance */
	  resch(j*n+i);
          i++; if (i%n==0) {i=0; j++; if (j>2) j=0;}
	  break;
        case '\010': case '\313':			/* backspace */
	  resch(j*n+i);
          if (i%n>0) i--; else if(j>0) {i=n-1; j--;}
	  break;
	case '<':					/* cursor to front */
	  resch(j*n+i);
	  i=0; j=0;
	  break;
	case '*':					/* cursor to center */
	  resch(j*n+i);
	  i=n/2; j=1;
	  break;
	case '>':					/* cursor to end */
	  resch(j*n+i);
	  i=n-1; j=2;
	  break;
	case '\322':					/* insert */
	  arule[j*n+i]='+';
	  resch(j*n+i);
          i++; if (i%n==0) {i=0; j++; if (j>2) j=0;}
	  break;
	case '\323':					/* delete */
	  arule[j*n+i]=' ';
	  resch(j*n+i);
          i++; if (i%n==0) {i=0; j++; if (j>2) j=0;}
	  break;
	case '\013': case '\310':			/* up arrow */
	  resch(j*n+i);
          if (j==0) j=2; else j--;
	  break;
	case '\012': case '\320':			/* down arrow */
	  resch(j*n+i);
          j++; if (j>2) j=0;
	  break;
	default:
	  break;
          }
    }
eran();
}

/* restore character in edsqrule */
resch(i) int i; {videoputc(sqrule[i],(arule[i]==' ')?1:2);}

/* show neighborhood in edsqrule */
/* i = center cell */
/* j = Golay code  */
shon(i,j) int i, j; {int k;
  videocursor(0,NRR,NRC);
  videoputc('0'+i,1);
  videocursor(0,NRR,NRC+1);
  k=j; videoputc('0'+k%KK,1);
  videocursor(0,NRR-1,NRC+1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR-1,NRC);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR-1,NRC-1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR,NRC-1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR+1,NRC-1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR+1,NRC);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR+1,NRC+1);
  videoputc('0'+k/KK,1);
}

/* erase the neighborhood in edsqrule */
eran() {
  videocursor(0,NRR,NRC); videoputc(' ',1);
  videocursor(0,NRR,NRC+1); videoputc(' ',1);
  videocursor(0,NRR-1,NRC+1); videoputc(' ',1);
  videocursor(0,NRR-1,NRC); videoputc(' ',1);
  videocursor(0,NRR-1,NRC-1); videoputc(' ',1);
  videocursor(0,NRR,NRC-1); videoputc(' ',1);
  videocursor(0,NRR+1,NRC-1); videoputc(' ',1);
  videocursor(0,NRR+1,NRC); videoputc(' ',1);
  videocursor(0,NRR+1,NRC+1); videoputc(' ',1);
}

/* edit the plane of cells */
edsqline(arr1) int arr1[QL][QL]; {
char c;
int  i, j, k, arr2[QL][QL];

for (i=0; i<QL; i++) for (j=0; j<QL; j++) arr2[i][j]=0;

//videoscroll(18,0,24,25,0,0);
//videocursor(0,19,0);
//scrstr("insert states with 0,1,.");
//videocursor(0,20,0);
//scrstr("move cursor using arrows");
//videocursor(0,21,0);
//scrstr("NSEW R CGO fgmxz *<>^_");
//videocursor(0,22,0);
//scrstr("carriage return exits");

while (0<1) {
  adot(li,lj,3);
//  videocursor(0,0,1);
//  videoputc('?',1);
//  c=kbdin();
//  videocursor(0,0,1);
//  videoputc(c,1);
  if (c == '\015') break;
  switch (c) {
    case '0': case '1':	case '.':	/* enter state */
      if (c=='.') c='1';
      i=c-'0';
      arr1[li][lj]=i;
      adot(li,lj,i);
      li++; if (li==QL) li=0;
      break;
    case '\012': case '\320':		/* downward */
      adot(li,lj,arr1[li][lj]);
      lj++; if (lj==QL) lj=0;
      break;
    case '\013': case '\310':		/* upward   */
      adot(li,lj,arr1[li][lj]);
      if (lj==0) lj=QL; lj--;
      break;
    case '\014': case '\315': case ' ':	/* forward  */
      adot(li,lj,arr1[li][lj]);
      li++; if (li==QL) li=0;
      break;
    case '\010': case '\313':		/* backward */
      adot(li,lj,arr1[li][lj]);
      if (li==0) li=QL; li--;
      break;
    case '<':				/* left margin     */
      adot(li,lj,arr1[li][lj]);
      li=0;
      break;
    case '>':				/* right margin    */
      adot(li,lj,arr1[li][lj]);
      li=QL-1;
      break;
    case '^':				/* top margin      */
      adot(li,lj,arr1[li][lj]);
      lj=0;
      break;
    case '_':				/* bottom margin   */
      adot(li,lj,arr1[li][lj]);
      lj=QL-1;
      break;
    case '*':				/* center of field */
      adot(li,lj,arr1[li][lj]);
      li=QL/2; 
      lj=QL/2;
      break;
    case 'a':				/* insert glider gun */
      lcell(1,4); lcell(1,5);
      lcell(2,4); lcell(2,5);
      lcell(12,3); lcell(12,4); lcell(12,5);
      lcell(13,2); lcell(13,6);
      lcell(14,1); lcell(14,7);
      lcell(15,2); lcell(15,6);
      lcell(16,3); lcell(16,4); lcell(16,5);
      lcell(17,3); lcell(17,4); lcell(17,5);
      lcell(22,5); lcell(22,6); lcell(22,7);
      lcell(23,4); lcell(23,5); lcell(23,7); lcell(23,8);
      lcell(24,4); lcell(24,5); lcell(24,7); lcell(24,8);
      lcell(25,4); lcell(25,5); lcell(25,6); lcell(25,7); lcell(25,8);
      lcell(26,3); lcell(26,4); lcell(26,8); lcell(26,9);
      lcell(35,6); lcell(35,7);
      lcell(36,6); lcell(36,7);
      screen(arr1);
      break; 
    case 'b':				/* insert new gun */
      lcell(1,25); lcell(1,26); lcell(1,32); lcell(1,33);
      lcell(2,25); lcell(2,26); lcell(2,32); lcell(2,33);
      lcell(14,27); lcell(14,28); lcell(14,30); lcell(14,31);
      lcell(15,26); lcell(15,32);
      lcell(17,25); lcell(17,33);
      lcell(18,25); lcell(18,28); lcell(18,30); lcell(18,33);
      lcell(19,25); lcell(19,26); lcell(19,27); lcell(19,31); lcell(19,32); lcell(19,33);
      lcell(24,17);
      lcell(25,0); lcell(25,1); lcell(25,17); lcell(25,18);
      lcell(26,0); lcell(26,1); lcell(26,18); lcell(26,19);
      lcell(27,17); lcell(27,18);
      lcell(31,17); lcell(31,18);
      lcell(32,0); lcell(32,1); lcell(32,18); lcell(32,19); lcell(32,27); lcell(32,28);
      lcell(33,0); lcell(33,1); lcell(33,17); lcell(33,18); lcell(33,27); lcell(33,28);
      lcell(34,17);
      screen(arr1);
      break;
    case 'f':				/* generate a phrame */
      phrame (2);
      break;
    case 'g':				/* one cycle of evolution */
      phrame (1);
      onesqgen(arr1);
      screen(arr1);
      break;
    case 'm':				/* mark neighborhood */
      arule[golcl(li,lj,arr1)]='+';
      sqscrrul(TRR,TRC);
      break;
    case 'M':				/* mark neighborhood over a region */
      if (li<10||li>QL-11) break;
      if (lj<10||lj>QL-11) break;
      for (i=li-10; i<li+10; i++) {
      for (j=lj-10; j<lj+10; j++) {
        arule[golcl(i,j,arr1)]='+';
        }}
      sqscrrul(TRR,TRC);
      break;
    case 'x':				/* uncomitted transitions */
      for (k=0; k<YS; k++) arule[k]=' ';
      videocursor(0,TRR,TRC);
      sqscrrul(TRR,TRC);
      break;
    case 'z':				/* clear array  */
      zrarry(arr1);
      screen(arr1);
      break;
    case 'N':				/* rotate array up */
      for (i=0; i<QL; i++) {
        k=arr1[i][0]; 
        for (j=1; j<QL; j++) arr1[i][j-1]=arr1[i][j];
	arr1[i][QL-1]=k;
	}
      if (lj==0) lj=QL; lj--;
      screen(arr1);
      break;
    case 'S':				/* rotate array down */
      for (i=0; i<QL; i++) {
	k=arr1[i][QL-1];
        for (j=1; j<QL; j++) arr1[i][QL-j]=arr1[i][QL-j-1];
	arr1[i][0]=k;
	}
      lj++; if (lj==QL) lj=0;
      screen(arr1);
      break;
    case 'E':				/* rotate array right */
      for (j=0; j<QL; j++) {
	k=arr1[QL-1][j];
        for (i=1; i<QL; i++) arr1[QL-i][j]=arr1[QL-i-1][j];
	arr1[0][j]=k;
	}
      li++; if (li==QL) li=0;
      screen(arr1);
      break;
    case 'W':				/* rotate array left */
      for (j=0; j<QL; j++) {
	k=arr1[0][j];
        for (i=1; i<QL; i++) arr1[i-1][j]=arr1[i][j];
	arr1[QL-1][j]=k;
	}
      if (li==0) li=QL; li--;
      screen(arr1);
      break;
    case 'R':				/* reflect array in diagonal */
      for (j=1; j<QL; j++) {
      for (i=0; i<j;  i++) {
        k=arr1[i][j];
	arr1[i][j]=arr1[j][i];
        arr1[j][i]=k;
        }}
      k=li; li=lj; lj=k;
      screen(arr1);
      break;
    case 'C':				/* save cell array */
      for (j=0; j<QL; j++) {
      for (i=0; i<QL; i++) {
	arr2[i][j]=arr1[i][j];
	}}
      break;
    case 'G':				/* recover saved cell array */
      for (j=0; j<QL; j++) {
      for (i=0; i<QL; i++) {
	arr1[i][j]=arr2[i][j];
	}}
      screen(arr1);
      break;
    case 'O':				/* or saved cell array */
      for (j=0; j<QL; j++) {
      for (i=0; i<QL; i++) {
	arr1[i][j]|=arr2[i][j];
	}}
      screen(arr1);
      break;
    default:
      break;
      }				/* end switch */
  }					/* end while */
adot(li,lj,arr1[li][lj]);
videoscroll(19,0,24,25,0,0);
videocursor(0,0,1);
videoputc(' ',1);
}					/* end edsqline */

/* generate the transitions for Conway's Life */
conlif() {int i;
  for (i=0; i<YS; i++) sqrule[i]='0';
  sqrule[5]='1';  sqrule[8]='1';  sqrule[9]='1';  sqrule[10]='1';
  sqrule[13]='1'; sqrule[14]='1'; sqrule[19]='1'; sqrule[20]='1';
  sqrule[22]='1'; sqrule[23]='1'; sqrule[56]='1'; sqrule[59]='1';
  sqrule[60]='1'; sqrule[61]='1'; sqrule[64]='1'; sqrule[65]='1';
  sqrule[70]='1'; sqrule[71]='1'; sqrule[73]='1'; sqrule[74]='1';
  sqrule[54]='1'; sqrule[55]='1'; sqrule[57]='1'; sqrule [58]='1';
  sqrule[63]='1'; sqrule[69]='1';
printf("conlif\n");
}

/* generate random cells throughout ar */
ransqarr(ar) int  ar[QL][QL]; {int i0, i1, i, c;
i=0;
for (i0=0; i0<QL; i0++) {
for (i1=0; i1<QL; i1++) {
  if (i%8 == 0) c=rand(); i++;
  ar[i0][i1]=c%KK; c/=KK;
  }}
}

/* generate random cells in the center of ar */
ranhcen(ar) int  ar[QL][QL]; {int i0, i1, ia, ib, i, c;
i=0;
for (i0=0; i0<QL; i0++)
for (i1=0; i1<QL; i1++)
  ar[i0][i1]=0;
ia=QL/3; ib=2*(QL/3);
for (i0=ia; i0<ib; i0++)
for (i1=ia; i1<ib; i1++) {
  if (i%8 == 0) c=rand(); i++;
  ar[i0][i1]=c%KK; c/=KK;
  }
}

/* generate a random rule */
ransqrul() {int i0, i, c;
i=0;
for (i0=0; i0<YS; i0++) {
  if (i%8 == 0) c=rand(); i++;
  sqrule[i0]=c%KK+'0'; c/=KK;
}
}

# include "nxruin21.c"

/* end nxsqul.c */
